/********************************************************
 * infinite-array -- routines to handle infinite arrays *
 *                                                      *
 * An infinite array is an array that grows as needed.  *
 * There is no index too large for an infinite array    *
 * (unless we run out of memory).                       *
 ********************************************************/
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

#include "ia.h"                 // get common definitions 

/*
 * the following three variables implement
 * a very simple pointer cache
 *
 * They store information on the last infinite array used,
 *       the last bucket we had and the last index used
 *
 * They are initialized to values we know will never
 * be used by any real array.
 */
static class infinite_array *cache_array_ptr = NULL;
static class infinite_array *cache_bucket_ptr = NULL;
static int cache_index = -1;
/********************************************************
 * operator [] -- find an element of an infinite array  *
 *                                                      *
 * Parameters                                           *
 *      index   -- index into the array                 *
 *                                                      *
 * Returns                                              *
 *      Reference to the element in the array		*
 ********************************************************/
int &infinite_array::operator [] (const int index)
{
    // pointer to the current bucket 
    class infinite_array *current_ptr;

    int current_index;	// Index we are working with

    // see if the cache will do us any good 
    if ((cache_array_ptr == this) && (index >= cache_index)) {
        current_ptr = cache_bucket_ptr;
        current_index = index - cache_index;
    } else {
        current_ptr = this;
        current_index = index;
    }

    while (current_index >= BLOCK_SIZE) {
        if (current_ptr->next == NULL) {
            current_ptr->next = new infinite_array;
            if (current_ptr->next == NULL) {
                cerr << "Error:Out of memory\n";
                exit(8);
            }
        }
        current_ptr = current_ptr->next;
        current_index -= BLOCK_SIZE;
    }

    cache_index = index - (index % BLOCK_SIZE);
    cache_array_ptr = this;
    cache_bucket_ptr = current_ptr;

    return (current_ptr->data[current_index]);
}

/********************************************************
 * ~infinite_array -- Destroy the infinite array	*
 ********************************************************/
infinite_array::~infinite_array(void)
{
    /*
     * Note: We use a cute trick here.
     *
     * Since each bucket in the infinite array 
     * an infinite array itself, when we destroy
     * next, it will destroy all it's "next"s
     * and so on recusively clearing the entire array.
     */
    if (next != NULL) {
       delete next;
       next = NULL;
    }
}
