/********************************************************
 * Stack						*
 *	A file implementing a simple stack class	*
 ********************************************************/
#include <stdlib.h>
#include <iostream.h>
#include <string.h>

const int STACK_SIZE = 100;	// Maximum size of a stack

/********************************************************
 * Stack class						*
 *							*
 * Member functions					*
 *	stack -- initalize the stack.			*
 *	push -- put an item on the stack.		*
 *	pop -- remove an item from the stack.		*
 ********************************************************/
// The stack itself
template<class kind>
class stack {
    private:
	int count;		// Number of items in the stack
	kind data[STACK_SIZE];	// The items themselves
    public:
	// Initialize the stack
	stack(void);

	// Push an item on the stack
	void push(const kind item);

	// Pop an item from the stack
	kind pop(void);
};

class stack<double> {
    private:
	int count;		// Number of items in the stack
	double data[STACK_SIZE];	// The items themselves
    public:
	// Initialize the stack
	stack(void);

	// Push an item on the stack
	void push(const double item);

	// Pop an item from the stack
	double pop(void);
};

/********************************************************
 * stack::stack -- initialize the stack.		*	
 ********************************************************/
template<class kind>
inline stack<kind>::stack(void)
{
    count = 0;	// Zero the stack
}
/********************************************************
 * stack::push -- push an item on the stack.		*
 *							*
 * Warning: We do not check for overflow.		*
 *							*
 * Parameters						*
 *	item -- item to put in the stack		*
 ********************************************************/
inline void stack<char *>::push(char * const item)
{
    data[count] = strdup(item);
    ++count;
}
/********************************************************
 * stack::pop -- get an item off the stack.		*
 *							*
 * Warning: We do not check for stack underflow.	*
 *							*
 * Returns						*
 * 	The top item from the stack.			*
 *							*
 * Note: Special version to override the default.	*
 *							*
 * Warning: Caller is expected to delete the item	*
 *	returned.  If you don't you will have a memory	*
 *	leak.						*
 ********************************************************/
inline char * stack<char *>::pop(void)
{
    // Stack goes down by one
    --count;

    // Then we return the top value
    return (data[count]);
}
/********************************************************
 * stack::push -- push an item on the stack.		*
 *							*
 * Warning: We do not check for overflow.		*
 *							*
 * Parameters						*
 *	item -- item to put in the stack		*
 ********************************************************/
template<class kind>
inline void stack<kind>::push(const kind item)
{
    data[count] = item;
    ++count;
}
/********************************************************
 * stack::pop -- get an item off the stack.		*
 *							*
 * Warning: We do not check for stack underflow.	*
 *							*
 * Returns						*
 * 	The top item from the stack.			*
 *							*
 * Note: Special version to override the default.	*
 ********************************************************/
inline float stack<float>::pop(void)
{
    // Stack goes down by one
    --count;

    // Then we return the top value
    return (data[count]*2);
}
/********************************************************
 * stack::pop -- get an item off the stack.		*
 *							*
 * Warning: We do not check for stack underflow.	*
 *							*
 * Returns						*
 * 	The top item from the stack.			*
 ********************************************************/
template<class kind>
inline kind stack<kind>::pop(void)
{
    // Stack goes down by one
    --count;

    // Then we return the top value
    return (data[count]);
}

// A short routine to test the stack
main()
{
    stack<int> a_stack;	// Stack we want to use
    stack<float> b_stack; // Another stack
    stack<char *> c_stack;// Character string class

    // Push three value on the stack
    a_stack.push(1);
    a_stack.push(2);
    a_stack.push(3);

    // Pop the item from the stack
    cout << "Expect a 3 ->" << a_stack.pop() << '\n';
    cout << "Expect a 2 ->" << a_stack.pop() << '\n';
    cout << "Expect a 1 ->" << a_stack.pop() << '\n';

    b_stack.push(1.0);
    cout << "Expect a 2 ->" << b_stack.pop() << '\n';

    c_stack.push("First");
    cout << "Expect a 'First' ->" << c_stack.pop() << '\n';

    return (0);
}
